﻿using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Problems.Instances;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  public class MISCInstanceProvider : ProblemInstanceProvider<MOTFData> {

    public override string Name {
      get { return "Miscellaneous"; }
    }

    public override string Description {
      get { return "Miscellaneous Testfunctions"; }
    }

    public override Uri WebLink {
      get { return new Uri("http://dev.heuristiclab.com/trac.fcgi/wiki/Documentation/Reference/Test%20Functions"); }
    }

    public override string ReferencePublication {
      get { return String.Empty; }
    }

    public override IEnumerable<IDataDescriptor> GetDataDescriptors() {
      var otherProviders = ApplicationManager.Manager.GetInstances <IProblemInstanceProvider<MOTFData>>().Where(x => x.Name != Name);

      var evaluators = ApplicationManager.Manager.GetInstances<IMultiObjectiveTestFunction>()
                                                 .Where(x => !handles(otherProviders,x))
                                                 .OrderBy(x => x.Name);
      return evaluators.Select(x => new MOTFDataDescriptor(x));
    }

    private bool handles(IEnumerable<IProblemInstanceProvider<MOTFData>> others, IMultiObjectiveTestFunction x) {
      foreach (var o in others) if (x.Name.StartsWith(o.Name)) return true;
      return false;
    }

    public override MOTFData LoadData(IDataDescriptor id) {
      var descriptor = (MOTFDataDescriptor)id;
      return new MOTFData {
        Name = descriptor.Name,
        Description = descriptor.Description,
        Evaluator = descriptor.Evaluator
      };
    }
  }
}
