﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [Item("ZDT2", "ZDT2 function as defined in http://www.tik.ee.ethz.ch/sop/download/supplementary/testproblems/ [30.11.2015]")]
  [StorableClass]
  public class ZDT2 : ZDT {
                                                                   
    public override double BestKnownHypervolume(int objectives) {
      return 120+1.0/3;
    }

    [StorableConstructor]
    protected ZDT2(bool deserializing) : base(deserializing) { }
    protected ZDT2(ZDT2 original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ZDT2(this, cloner);
    }
    public ZDT2() : base() { }

    public override double[] Evaluate(RealVector r) {
      double g = 0;
      for (int i = 1; i < r.Length; i++) g += r[i];
      g = 1.0 + 9.0 * g / (r.Length - 1);
      double d = r[0] / g;
      return new double[] { r[0], g * (1.0 - d * d) };
    }
  }
}
